package com.simonstl.fragment;

import java.util.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>Each FragmentRule describes a regular expression and the results of the fragmentation
* it produces, described as a series of child elements.</p>
*
* <p>Currently, the series is a simple list.  Eventually it may provide for collections
* (appropriate for the HTML style element and some SVG) and for attribute results.</p>
*
* <p>Rules currently don't keep track of which elements they apply to. That information is stored
* in the {@link com.simonstl.fragment.FragmentRules FragmentRules} element. 
* This may change in future versions.</p>
*
* <p>Version 0.08 adds better print() and toString() support.</p>
*
* <p>Version 0.07 now returns {@link com.simonstl.fragment.DocComponent DocComponent} objects more effectively, uses the <code>cleanDuplicate()</code> method to ensure that object reuse doesn't mar results.</p>
*
* <p>Version 0.06 moved regular expression processing to the RegExProc (now {@link com.simonstl.fragment.RegexProcFactory RegexProcFactory}) 
* class, a first step toward permitting the use of pluggable regular expressions processors.</p>
*
* <p>Version 0.05 adds support for DocComponents rather than Elements, genericizing processing a bit.</p>
*
* <p>Version 0.04 moved regular expression processing from the {@link com.simonstl.fragment.FragmentFilter FragmentFilter}
* into the processRule() method.</p>
*
* <p>Version 0.03 adds support for the split attribute.</p>
*
* <p>Version 0.02 adds information about repetition and skipping the first result.  By default,
* rules don't repeat and are assumed to skip the first result.  If the <code>repeat</code> attribute
* of the <code>fragmentRule</code> element is set to "<code>true</code>", the rule will cycle through
* the elements in the <code>produces</code> element rather than just proceeding once.  Similarly, setting
* the <code>skipFirst</code> attribute to "<code>false</code> will preserve the first match on the
* regular expression.</p>
*
* @version 0.08 4 July 2001
* @author Simon St.Laurent
*/


public class FragmentRule {
    public String pattern;
    public ArrayList results;
    public boolean repeats=false;
    public boolean skipFirst=true;
    final int MATCH=0;
    final int SPLIT=1;
    final int SERIES=2;
    public int isSplit=MATCH;
    final int ELEMENTS_ONLY=0;
    final int SINGLETONS_ONLY=1;
    final int ALL=2;
    private int matchLevel=ALL;
    private RegExInt regExProc=null;

public FragmentRule (String regEx) {
    pattern=regEx;
    results=new ArrayList();
}

public FragmentRule (String regEx, ArrayList endResults) {
    pattern=regEx;
    results=endResults;
}
public FragmentRule (String regEx, ArrayList endResults, boolean doesRepeat) {
    pattern=regEx;
    results=endResults;
    repeats=doesRepeat;
}


public void complete () {
    regExProc=RegexProcFactory.createRegEx(isSplit, pattern);
}

public void setRepeat(boolean doesRepeat) {
    repeats=doesRepeat;
}
public boolean getRepeat() {
    return repeats;
}

public void setSkipFirst(boolean skip) {
    skipFirst=skip;
}
public boolean getSkipFirst() {
    return skipFirst;
}

public void setIsSplit(int split) {
    isSplit=split;
// System.out.println("Set isSplit to: "+isSplit);
}
public int getIsSplit() {
    return isSplit;
}

public int getSize(){
    return results.size();
}

public void setMatchLevel(int level) {
   matchLevel=level;
}
public DocComponent getResults(int which) {
  if (repeats) {
    int currentSize=results.size();
//System.out.println("start which: "+which);
//System.out.println("current size: " + currentSize);
    which=which%currentSize;
    if (which<0) {which=which+currentSize;}
//System.out.println("which: " + which);
  }
  return (DocComponent) results.get(which);
}

public ArrayList processResult(String elementContent) {
//System.out.println("Processing");
 int start=0;
 ArrayList newResults=new ArrayList();
   if (skipFirst) {
     start=1;
   }
//System.out.println("regex: "+pattern);

   ArrayList stringResults=regExProc.divide(elementContent);
     int numberMatches=stringResults.size();
//System.out.println(numberMatches-1+" matches, start="+start);
     for (int i=start; i<numberMatches; i++) {
       String value=new String();
       value=(String) stringResults.get(i);
       DocComponent currentComponent=(DocComponent)results.get(i-start);
  //copy the result element to a new element with the content from the match!
       DocComponent newComponent=(DocComponent) currentComponent.cleanDuplicate();
       newComponent.setContent(value);
//newComponent.print();
       newResults.add(newComponent);
  // then the fragment filter will have to tell the results to call it recursively
      }//end for
   return newResults;
}

public void print() {
System.out.println(toString());
}

public String toString(){
  String value="";
  int size=results.size();
  for (int i=0; i<size; i++){
    DocComponent currentComponent=(DocComponent)results.get(i);
    String currentComponentName=currentComponent.toString();
    value=value+("produces: " + currentComponentName+"\n");
  }
return value;
}

public void addResult (DocComponent resultComponent) {
    results.add(resultComponent);
}


}